#!/bin/perl
#
# Example code to scan Active Directory for user group membership
# using Perl and Net::LDAP.
#
# Usage: perl ADGroupsExample.pl
#
# Shawn Poulson, 2009.05.18
# http://www.explodingcoder.com
#

use Net::LDAP;

my ($ldap_server, $ldap_username, $ldap_password) =
   ('<LDAP hostname>', '<service_account>', '<password>');

# Login to LDAP
print "Connecting to LDAP...";
my $ldap = Net::LDAP->new($ldap_server, async => 0) or die $@;
print "Binding... ";
$_ = $ldap->bind($ldap_username, password => $ldap_password) or die $@;
print $_->error_text();

# Get currently logged in user
my $userDN = GetDNByID($ldap, $ENV{USERNAME});
print "User DN: $userDN\n";

# Quick check if user is a member of a group
print "User is a member of 'IT': ", IsMemberOf($ldap, $userDN, GetDNByID($ldap, 'IT')) ? 'True' : 'False', "\n";
print "User is a member of 'Foobar': ", IsMemberOf($ldap, $userDN, GetDNByID($ldap, 'Foobar')) ? 'True' : 'False', "\n";

# Enumerate all member group names
my @tkUser = GetTokenGroups($ldap, $userDN); # Get tokens of member groups
print "User is a member of ", scalar @tkUser, " groups:\n";
foreach my $tk (@tkUser) {
   print "   ", GetIDBySid($ldap, $tk), "\n";
}

# Scan token list for groups
my $ITSid = GetSidByID($ldap, 'IT');
my $isMemberIT = !!scalar grep { $ITSid eq $_ } @tkUser;
print "User is a member of 'IT': ", $isMemberIT ? 'True' : 'False', "\n";

my $FoobarSid = GetSidByID($ldap, 'Foobar');
my $isMemberFoobar = !!scalar grep { $FoobarSid eq $_ } @tkUser;
print "User is a member of 'Foobar': ", $isMemberFoobar ? 'True' : 'False', "\n";

# Done
$ldap->unbind;
exit;


# =====================================
# Token query routines
# =====================================

# Is DN a member of security group?
# Usage: <bool> = IsMemberOf(<DN of object>, <DN of group>)
sub IsMemberOf($$$) {
   my ($ldap, $objectDN, $groupDN) = @_;
   return if ($groupDN eq "");

   my $groupSid = GetSidByDN($ldap, $groupDN);
   return if ($groupSid eq "");

   my @matches = grep { $_ eq $groupSid } GetTokenGroups($ldap, $objectDN);

   @matches > 0;
}

# Gets tokenGroups attribute from the provided DN
# Usage: <Array of tokens> = GetTokenGroups(<LDAP ref>, <DN of object>)
sub GetTokenGroups($$) {
   my ($ldap, $objectDN) = @_;
   
   my $results = $ldap->search(
      base => $objectDN,
      scope => 'base',
      filter => '(objectCategory=*)',
      attrs => ['tokenGroups']
   );

   if ($results->count) {
      return $results->entry(0)->get_value('tokenGroups');
   }
}


# =====================================
# Query helper routines
# =====================================

# Get object's SID by DN
# Usage: <SID> = GetSidByDN(<LDAP ref>, <DN>)
sub GetSidByDN($$) {
   my ($ldap, $objectDN) = @_;
   
   my $results = $ldap->search(
      base => $objectDN,
      scope => 'base',
      filter => '(objectCategory=*)',
      attrs => ['objectSid']
   );

   if ($results->count) {
      return $results->entry(0)->get_value('objectSid');
   }
}

# Get object's SID by sAMAccountName
# Usage: <SID> = GetSidByID(<LDAP ref>, <sAMAccountName>)
sub GetSidByID($$) {
   my ($ldap, $ID) = @_;

   my $results = $ldap->search(
      base => GetRootDN($ldap),
      filter => "(sAMAccountName=$ID)",
      attrs => ['objectSid']
   );

   if ($results->count) {
      return $results->entry(0)->get_value('objectSid');
   }
}

# Get DN by sAMAccountName
# Usage: <DN> = GetDNByID(<LDAP ref>, <ID>)
sub GetDNByID($$) {
   my ($ldap, $ID) = @_;
   
   my $results = $ldap->search(
      base => GetRootDN($ldap),
      filter => "(sAMAccountName=$ID)",
      attrs => ['distinguishedName']
   );

   if ($results->count) {
      return $results->entry(0)->get_value('distinguishedName');
   }
}

# Get sAMAccountName by object's SID
# Usage: <ID> = GetIDBySid(<LDAP ref>, <SID>)
sub GetIDBySid($$) {
   my ($ldap, $objectSid) = @_;
   
   my $results = $ldap->search(
      base => '<SID=' . unpack('H*', $objectSid) . '>',
      scope => 'base',
      filter => '(objectCategory=*)',
      attrs => ['sAMAccountName']
   );
   
   if ($results->count) {
      return $results->entry(0)->get_value('sAMAccountName');
   }
}

# =====================================
# LDAP routines
# =====================================

# Get Root DN of logged in domain (e.g. DC=yourdomain,DC=com)
# Usage: <DN> = GetRootDN(<LDAP ref>)
sub GetRootDN($) {
   my ($ldap) = @_;
   ($ldap->root_dse->get_value('namingContexts'))[0];
}
