'
' Example code to scan Active Directory for user group membership
' using VBScript under Windows Script Host and ADSI.
'
' Usage: cscript ADGroupsExample.vbs
'
' Shawn Poulson, 2009.05.18
' http://www.explodingcoder.com
'

' Get current logged in user info
Set oADSysInfo = CreateObject("ADSystemInfo")
userDN = oADSysInfo.UserName     ' Get DN of user
WScript.Echo "User DN: " & userDN

' Quick check if user is a member of a group
WScript.Echo "User is a member of 'IT': " & IsMemberOf(userDN, GetDNByID("IT"))
WScript.Echo "User is a member of 'Foobar': " & IsMemberOf(userDN, GetDNByID("Foobar"))

' Enumerate all member group names
tkUser = GetTokenGroups(userDN)  ' Get tokens of member groups
WScript.Echo "User is a member of " & (UBound(tkUser) + 1) & " groups:"
For Each tk in tkUser
   WScript.Echo "   " & GetIDBySid(tk)
Next

' Scan token list for groups
WScript.Echo "User is a member of 'IT': " & TokenListFindSid(tkUser, GetSidByID("IT"))
WScript.Echo "User is a member of 'Foobar': " & TokenListFindSid(tkUser, GetSidByID("Foobar"))

' Done


' =====================================
' Token query routines
' =====================================

' Is DN a member of security group?
' Usage: <bool> = IsMemberOf(<DN of object>, <DN of group>)
Function IsMemberOf(dnObject, dnGroup)
   IsMemberOf = TokenListFindSid(GetTokenGroups(dnObject), GetSidByDN(dnGroup))
End Function

' Gets tokenGroups attribute from the provided DN
' Usage: <Array of tokens> = GetTokenGroups(<DN of object>)
Function GetTokenGroups(dnObject)
   Dim adsObject

   ' Setup query of tokenGroup SIDs from dnObject
   Set adsObject = GetObject(LdapUri(dnObject))
   adsObject.GetInfoEx Array("tokenGroups"), 0
   GetTokenGroups = adsObject.GetEx("tokenGroups")
End Function

' Checks if the SID of a DN is found in an array of tokens.
' Usage: <bool> = TokenListFindSid(<Array of tokens>, <Object SID Byte()>)
Function TokenListFindSid(arrTokens, objectSid)
   Dim nSidSize, vSidHex, e
   TokenListFindSid = False
   
   If TypeName(objectSid) = "Byte()" Then
      ' Scan token array for object SID
      nSidSize = UBound(objectSid)
      vSidHex = ByteArrToHexString(objectSid)
      For Each e in arrTokens
         If UBound(e) = nSidSize Then
            If ByteArrToHexString(e) = vSidHex Then
               TokenListFindSid = True
               Exit For
            End If
         End If
      Next
   End If
End Function

' Encode Byte() to hex string
Function ByteArrToHexString(bytes)
   Dim i
   ByteArrToHexString = ""
   For i = 1 to Lenb(bytes)
      ByteArrToHexString = ByteArrToHexString & Right("0" & Hex(Ascb(Midb(bytes, i, 1))), 2)
   Next
End Function

' Format a DN into a valid LDAP URI
Function LdapUri(DN)
   LdapUri = "LDAP://" & Replace(DN, "/", "\/")
End Function


' =====================================
' Query helper routines
' =====================================

' Get object's SID by DN
' Usage: <SID Byte()> = GetSidByDN(<DN>)
Function GetSidByDN(objectDN)
   On Error Resume Next
   GetSidByDN = GetObject(LdapUri(objectDN)).Get("objectSid")
   On Error GoTo 0
End Function

' Get object's SID by ID
' Usage: <SID Byte()> = GetSidByID(<Object ID>)
Function GetSidByID(ID)
   Dim rs
   Set rs = QueryLDAP(GetRootDN, "(sAMAccountName=" & ID & ")", "objectSid", "subtree")
   If Not rs.EOF Then GetSidByID = rs("objectSid")
   rs.Close
   Set rs = Nothing
End Function

' Get DN by sAMAccountName
' Usage: <DN> = GetDNByID(<User or Group ID>)
Function GetDNByID(ID)
   Dim rs
   Set rs = QueryLDAP(GetRootDN, "(sAMAccountName=" & ID & ")", "distinguishedName", "subtree")
   If Not rs.EOF Then GetDNByID = rs("distinguishedName")
   rs.Close
   Set rs = Nothing
End Function

' Get sAMAccountName by object's SID
' Usage: <sAMAccountName> = GetIDBySid(<SID Byte()>)
Function GetIDBySid(objectSid)
   If TypeName(objectSid) = "Byte()" Then
      GetIDBySid = GetObject("LDAP://<SID=" & ByteArrToHexString(objectSid) & ">").Get("sAMAccountName")
   End If
End Function


' =====================================
' LDAP routines
' =====================================

' Get Root DN of logged in domain (e.g. DC=yourdomain,DC=com)
' Usage: <DN> = GetRootDN
Function GetRootDN
   GetRootDN = GetObject("LDAP://RootDSE").Get("defaultNamingContext")
End Function

' Get/create singleton LDAP ADODB connection object
' Usage: <Connection object ref> = LDAPConnection
'    or: LDAPConnection.<property or method>
Dim l_LDAPConnection
Function LDAPConnection
   If IsEmpty(l_LDAPConnection) Then
      Set l_LDAPConnection = CreateObject("ADODB.Connection")
      l_LDAPConnection.Provider = "ADSDSOObject"
      l_LDAPConnection.Open "ADs Provider"
   End If
   Set LDAPConnection = l_LDAPConnection
End Function

' Close the LDAPConnection singleton object
' Usage: CloseLDAPConnection
Sub CloseLDAPConnection
   If IsObject(l_LDAPConnection) Then
      If l_LDAPConnection.State = 1 Then l_LDAPConnection.Close
   End If
   l_LDAPConnection = Empty
End Sub

' Query LDAP helper, return RecordSet
' Usage: <RecordSet object ref> = QueryLDAP(<DN>, <LDAP Filter>, <Attributes CSV>, <Scope>
' Scope can be: "subtree", "onelevel", or "base"
' Be sure to close the RecordSet object when done with it
Function QueryLDAP(DN, Filter, AttributeList, Scope)
   Set QueryLDAP = LDAPConnection.Execute("<" & LdapUri(DN) & ">;" & Filter & ";" & AttributeList & ";" & Scope)
End Function
